@@ -1,26 +1,26 @@
-# -*- coding: utf-8 -*-
-"""
-"""
-
+from __future__ import annotations
 
 from prettyqt import constants, core, widgets
+from prettyqt.qt import QtCore
 
 
 class RegexMatchesModel(core.AbstractTableModel):
 
     HEADER = ["Start", "End", "Value", "Groups"]
 
-    def __init__(self, matches, parent=None):
+    def __init__(self, matches: list | None = None, parent: QtCore.QObject | None = None):
         super().__init__(parent=parent)
-        self.matches = matches
+        self.matches = matches if matches else list()
 
     def columnCount(self, parent=None):
         return len(self.HEADER)
 
-    def headerData(self, offset: int, orientation, role):
+    def headerData(  # type: ignore
+        self, section: int, orientation: QtCore.Qt.Orientation, role: int
+    ) -> str | None:
         if role == constants.DISPLAY_ROLE:
             if orientation == constants.HORIZONTAL:
-                return self.HEADER[offset]
+                return self.HEADER[section]
 
     def data(self, index, role):
         if not index.isValid():
@@ -39,9 +39,7 @@ class RegexMatchesModel(core.AbstractTableModel):
             return item.span()
 
     def rowCount(self, parent=None):
-        """
-        required override for AbstractitemModels
-        """
+        """Required override for AbstractitemModels."""
         return len(self.matches)
 
 
@@ -58,11 +56,11 @@ if __name__ == "__main__":
     matches = list(compiled.finditer(teststring))
     model = RegexMatchesModel(matches)
 
-    view.setModel(model)
+    view.set_model(model)
     dlg = widgets.Dialog(layout="horizontal")
     dlg.add_widget(view)
     dlg.show_blocking()
     view.resize(500, 300)
     print(view.model())
     print(view.model().rowCount())
-    app.exec_()
+    app.main_loop()
