@@ -1,8 +1,6 @@
-# -*- coding: utf-8 -*-
+"""Custom_validators module.
 
-"""custom_models module
-
-contains custom models
+Contains custom validators
 """
 
 
@@ -10,13 +8,17 @@ from .compositevalidator import CompositeValidator
 from .notemptyvalidator import NotEmptyValidator
 from .notzerovalidator import NotZeroValidator
 from .pathvalidator import PathValidator
+from .integervalidator import IntegerValidator
 from .regexvalidators import FloatListValidator, IntListValidator
 from .regexpatternvalidator import RegexPatternValidator
 
-__all__ = ["PathValidator",
-           "NotEmptyValidator",
-           "IntListValidator",
-           "FloatListValidator",
-           "NotZeroValidator",
-           "CompositeValidator",
-           "RegexPatternValidator"]
+__all__ = [
+    "PathValidator",
+    "NotEmptyValidator",
+    "IntegerValidator",
+    "IntListValidator",
+    "FloatListValidator",
+    "NotZeroValidator",
+    "CompositeValidator",
+    "RegexPatternValidator",
+]
