@@ -1,13 +1,9 @@
-# -*- coding: utf-8 -*-
-"""
-@author: Philipp Temminghoff
-based on http://www.yasinuludag.com/blog/?p=49
-"""
+"""Based on http://www.yasinuludag.com/blog/?p=49 ."""
+
+from __future__ import annotations
 
 from prettyqt import core, gui, syntaxhighlighters
 
-HIGHLIGHT_RULES = []
-
 
 class Rule(syntaxhighlighters.HighlightRule):
     pass
@@ -46,15 +42,15 @@ class Keyword(Rule):
 
 VALUE_FORMAT = gui.TextCharFormat()
 VALUE_FORMAT.set_foreground_color("orange")
-VALUE_START_EXPRESSION = core.RegExp(r"\"")
-VALUE_END_EXPRESSION = core.RegExp(r"\"(?=[\s></])")
+VALUE_START_EXPRESSION = core.RegularExpression(r"\"")
+VALUE_END_EXPRESSION = core.RegularExpression(r"\"(?=[\s></])")
 
 
 class XmlHighlighter(gui.SyntaxHighlighter):
 
     RULES = Rule.__subclasses__()
 
-    def highlightBlock(self, text):
+    def highlightBlock(self, text: str):
         super().highlightBlock(text)
         # HANDLE QUOTATION MARKS NOW.. WE WANT TO START WITH " AND END WITH "..
         # A THIRD " SHOULD NOT CAUSE THE WORDS INBETWEEN SECOND AND THIRD
@@ -62,23 +58,26 @@ class XmlHighlighter(gui.SyntaxHighlighter):
         self.setCurrentBlockState(0)
         start_index = 0
         if self.previousBlockState() != 1:
-            start_index = VALUE_START_EXPRESSION.indexIn(text)
+            start_index = VALUE_START_EXPRESSION.match(text).capturedStart()
         while start_index >= 0:
-            end_index = VALUE_END_EXPRESSION.indexIn(text, start_index)
+            match = VALUE_END_EXPRESSION.match(text, start_index)
+            end_index = match.capturedStart()
             if end_index == -1:
                 self.setCurrentBlockState(1)
                 comment_len = len(text) - start_index
             else:
-                matched_len = VALUE_END_EXPRESSION.matchedLength()
-                comment_len = end_index - start_index + matched_len
+                comment_len = end_index - start_index + match.capturedLength()
             self.setFormat(start_index, comment_len, VALUE_FORMAT)
-            start_index = VALUE_START_EXPRESSION.indexIn(text, start_index + comment_len)
+            start_index = VALUE_START_EXPRESSION.match(
+                text, start_index + comment_len
+            ).capturedStart()
 
 
 if __name__ == "__main__":
     from prettyqt import widgets
+
     app = widgets.app()
     editor = widgets.PlainTextEdit()
     highlighter = XmlHighlighter(editor.document())
     editor.show()
-    app.exec_()
+    app.main_loop()
