@@ -1,10 +1,7 @@
-# -*- coding: utf-8 -*-
-"""
-@author: Philipp Temminghoff
-"""
+from __future__ import annotations
 
-from qtpy import QtWidgets
 from prettyqt import widgets
+from prettyqt.qt import QtWidgets
 
 
 QtWidgets.QColumnView.__bases__ = (widgets.AbstractItemView,)
@@ -15,8 +12,8 @@ class ColumnView(QtWidgets.QColumnView):
 
 
 if __name__ == "__main__":
-    app = QtWidgets.QApplication([])
-    dlg = QtWidgets.QMainWindow()
+    app = widgets.app()
+    dlg = widgets.MainWindow()
     status_bar = ColumnView()
     dlg.show()
-    app.exec_()
+    app.main_loop()
