@@ -1,29 +1,40 @@
-# -*- coding: utf-8 -*-
-"""
-"""
+from __future__ import annotations
 
-from typing import Optional
+from typing import Literal
 
-from qtpy import QtCore, QtGui, QtWidgets
-
-from prettyqt import core, gui, widgets
-from prettyqt.utils import bidict, icons
+from prettyqt import core, gui, iconprovider, widgets
+from prettyqt.qt import QtCore, QtGui, QtWidgets
+from prettyqt.utils import InvalidParamError, bidict, types
 
 
 TAB_SHAPES = bidict(
-    rounded=QtWidgets.QTabWidget.Rounded, triangular=QtWidgets.QTabWidget.Triangular
+    rounded=QtWidgets.QTabWidget.TabShape.Rounded,
+    triangular=QtWidgets.QTabWidget.TabShape.Triangular,
 )
 
+TabShapeStr = Literal["rounded", "triangular"]
+
+TAB_POSITION = bidict(
+    north=QtWidgets.QTabWidget.TabPosition.North,
+    south=QtWidgets.QTabWidget.TabPosition.South,
+    west=QtWidgets.QTabWidget.TabPosition.West,
+    east=QtWidgets.QTabWidget.TabPosition.East,
+)
+
+TabPositionStr = Literal["north", "south", "west", "east"]
 
 QtWidgets.QTabWidget.__bases__ = (widgets.Widget,)
 
 
 class TabWidget(QtWidgets.QTabWidget):
-    """
-    Widget for managing the tabs section
-    """
+    """Widget for managing the tabs section."""
 
-    def __init__(self, parent=None, closable=False, detachable=False):
+    def __init__(
+        self,
+        parent: QtWidgets.QWidget | None = None,
+        closable: bool = False,
+        detachable: bool = False,
+    ) -> None:
 
         # Basic initalization
         super().__init__(parent)
@@ -34,21 +45,27 @@ class TabWidget(QtWidgets.QTabWidget):
 
         # Used to keep a reference to detached tabs since their QMainWindow
         # does not have a parent
-        self.detached_tabs = dict()
+        self.detached_tabs: dict[str, DetachedTab] = {}
         if detachable:
             self.set_detachable()
         self.set_closable(closable)
 
-    def __len__(self):
+    def __len__(self) -> int:
         return self.count()
 
-    def __getitem__(self, index):
+    def __getitem__(self, index: int) -> QtWidgets.QWidget:
         if isinstance(index, int):
             return self.widget(index)
         else:
-            return self.findChild(QtWidgets.QWidget, index)
+            result = self.findChild(QtWidgets.QWidget, index)
+            if result is None:
+                raise KeyError("Widget not found")
+            return result
 
-    def __getstate__(self):
+    def __contains__(self, item: QtWidgets.QWidget):
+        return self.indexOf(item) >= 0
+
+    def serialize_fields(self):
         return dict(
             tabbar=self.tabBar(),
             widgets=self.get_children(),
@@ -58,10 +75,11 @@ class TabWidget(QtWidgets.QTabWidget):
             tab_shape=self.get_tab_shape(),
             # elide_mode=self.get_elide_mode(),
             icon_size=self.iconSize(),
+            tab_position=self.get_tab_position(),
         )
 
     def __setstate__(self, state):
-        self.__init__()
+        super().__setstate__(state)
         self.setTabBar(state["tabbar"])
         self.setDocumentMode(state.get("document_mode", False))
         self.setMovable(state.get("movable", False))
@@ -73,35 +91,71 @@ class TabWidget(QtWidgets.QTabWidget):
             self.setTabWhatsThis(i, whatsthis)
         self.setCurrentIndex(state.get("index", 0))
 
-    def set_document_mode(self, state: bool = True):
+    def __reduce__(self):
+        return type(self), (), self.__getstate__()
+
+    def update_tab_bar_visibility(self):
+        """Update visibility of the tabBar depending of the number of tabs.
+
+        0 or 1 tab -> tabBar hidden, 2+ tabs - >tabBar visible
+        need to be called explicitly, or be connected to tabInserted/tabRemoved
+        """
+        self.tabBar().setVisible(self.count() > 1)
+
+    def set_icon_size(self, size: int | types.SizeType):
+        """Set size of the icons."""
+        if isinstance(size, int):
+            size = core.Size(size, size)
+        elif isinstance(size, tuple):
+            size = core.Size(*size)
+        self.setIconSize(size)
+
+    def set_document_mode(self, state: bool = True) -> None:
         self.setDocumentMode(state)
 
-    def set_tab_shape(self, shape: str):
-        """set tab shape for the tabwidget
-
-        Valid values are "rounded" and "triangular"
+    def set_tab_shape(self, shape: TabShapeStr):
+        """Set tab shape for the tabwidget.
 
         Args:
             shape: tab shape to use
 
         Raises:
-            ValueError: tab shape does not exist
+            InvalidParamError: tab shape does not exist
         """
         if shape not in TAB_SHAPES:
-            raise ValueError("Invalid value for shape.")
+            raise InvalidParamError(shape, TAB_SHAPES)
         self.setTabShape(TAB_SHAPES[shape])
 
-    def get_tab_shape(self) -> str:
-        """returns tab shape
-
-        possible values are "roundes", "triangular"
+    def get_tab_shape(self) -> TabShapeStr:
+        """Return tab shape.
 
         Returns:
             tab shape
         """
-        return TAB_SHAPES.inv[self.tabShape()]
+        return TAB_SHAPES.inverse[self.tabShape()]
 
-    def get_children(self) -> list:
+    def set_tab_position(self, position: TabPositionStr):
+        """Set tab position for the tabwidget.
+
+        Args:
+            position: tab position to use
+
+        Raises:
+            InvalidParamError: tab position does not exist
+        """
+        if position not in TAB_POSITION:
+            raise InvalidParamError(position, TAB_POSITION)
+        self.setTabPosition(TAB_POSITION[position])
+
+    def get_tab_position(self) -> TabPositionStr:
+        """Return tab position.
+
+        Returns:
+            tab position
+        """
+        return TAB_POSITION.inverse[self.tabPosition()]
+
+    def get_children(self) -> list[tuple]:
         return [
             (
                 self.widget(i),
@@ -113,30 +167,37 @@ class TabWidget(QtWidgets.QTabWidget):
             for i in range(self.count())
         ]
 
-    def tab_icon(self, i: int) -> gui.Icon:
-        return gui.Icon(self.tabIcon(i))
+    def tab_icon(self, i: int) -> gui.Icon | None:
+        icon = self.tabIcon(i)
+        if icon.isNull():
+            return None
+        return gui.Icon(icon)
 
     def set_detachable(self):
         self.tab_bar.on_detach.connect(self.detach_tab)
-        widgets.app().aboutToQuit.connect(self.close_detached_tabs)
+        core.CoreApplication.call_on_exit(self.close_detached_tabs)
         self.setMovable(True)
 
     def set_closable(self, closable: bool = True):
         self.setTabsClosable(closable)
 
     @core.Slot(int, QtCore.QPoint)
-    def detach_tab(self, index: int, point: QtCore.QPoint):
-        """
-        Detach the tab by removing it's contents and placing them in
-        a DetachedTab window
+    def detach_tab(self, index: int, point: types.PointType):
+        """Detach tab by removing its contents and placing them in a DetachedTab window.
 
-        @param index    index location of the tab to be detached
-        @param point    screen pos for creating the new DetachedTab window
+        Args:
+            index (int): index location of the tab to be detached
+            point (QtCore.QPoint): screen pos for creating the new DetachedTab window
+
+        Returns:
+            None: Description
         """
         # Get the tab content
+        if isinstance(point, tuple):
+            point = QtCore.QPoint(*point)
         name = self.tabText(index)
-        icon = self.tabIcon(index)
-        if icon.isNull():
+        icon = self.tab_icon(index)
+        if icon is None:
             icon = self.window().windowIcon()
         widget = self.widget(index)
 
@@ -159,12 +220,12 @@ class TabWidget(QtWidgets.QTabWidget):
 
     def add_tab(
         self,
-        item,
+        item: QtWidgets.QWidget | QtWidgets.QLayout,
         label: str,
-        icon: icons.IconType = None,
-        position: Optional[int] = None,
+        icon: types.IconType = None,
+        position: int | None = None,
         show: bool = False,
-    ):
+    ) -> int:
         if isinstance(item, QtWidgets.QLayout):
             widget = widgets.Widget()
             widget.set_layout(item)
@@ -175,7 +236,7 @@ class TabWidget(QtWidgets.QTabWidget):
         if not icon:
             index = self.insertTab(position, widget, label)
         else:
-            icon = icons.get_icon(icon)
+            icon = iconprovider.get_icon(icon)
             index = self.insertTab(position, widget, icon, label)
         if show:
             self.setCurrentIndex(index)
@@ -183,21 +244,24 @@ class TabWidget(QtWidgets.QTabWidget):
 
     def attach_tab(
         self,
-        widget,
+        widget: QtWidgets.QWidget | QtWidgets.QLayout,
         name: str,
-        icon: icons.IconType = None,
-        insert_at: Optional[int] = None,
+        icon: types.IconType = None,
+        insert_at: int | None = None,
     ):
-        """
+        """Re-attach tab.
+
         Re-attach the tab by removing the content from the DetachedTab window,
-        closing it, and placing the content back into the DetachableTabWidget
+        closing it, and placing the content back into the DetachableTabWidget.
 
-        @param    widget    the content widget from the DetachedTab window
-        @param    name             the name of the detached tab
-        @param    icon             the window icon for the detached tab
-        @param    insert_at         insert the re-attached tab at the given index
+        Args:
+            widget (Union[QtWidgets.QWidget, QtWidgets.QLayout]): the content widget
+                from the DetachedTab window
+            name (str): the name of the detached tab
+            icon (types.IconType, optional): the window icon for the detached tab
+            insert_at (Optional[int], optional): insert the re-attached tab at the
+                given index
         """
-
         widget.setParent(self)
 
         # Remove the reference
@@ -208,10 +272,10 @@ class TabWidget(QtWidgets.QTabWidget):
         self.add_tab(widget, name, icon=icon, position=insert_at, show=True)
 
     def close_detached_tabs(self):
-        """Close all tabs that are currently detached
-        """
-        for detached_tab in self.detached_tabs.values():
-            detached_tab.close()
+        """Close all tabs that are currently detached."""
+        tabs = list(self.detached_tabs.values())
+        for tab in tabs:
+            tab.close()
 
     @core.Slot(int)
     def remove_tab(self, index: int):
@@ -220,19 +284,17 @@ class TabWidget(QtWidgets.QTabWidget):
         if widget is not None:
             widget.deleteLater()
 
-    @core.Slot(object, str)
+    @core.Slot(QtWidgets.QWidget, str)
     def open_widget(self, widget: QtWidgets.QWidget, title: str = "Unnamed"):
-        """
-        create a tab containing delivered widget
-        """
+        """Create a tab containing delivered widget."""
         self.add_tab(widget, title, icon="mdi.widgets", show=True)
 
-    def set_tab(self, index, position: str, widget=None):
+    def set_tab(self, index: int, position: str, widget: QtWidgets.QWidget | None = None):
         self.tabBar().set_tab(index, position, widget)
 
 
 class DetachedTab(widgets.MainWindow):
-    """window containing a detached tab
+    """Window containing a detached tab.
 
     When a tab is detached, the contents are placed into this QMainWindow.
     The tab can be re-attached by closing the dialog
@@ -243,7 +305,7 @@ class DetachedTab(widgets.MainWindow):
 
     on_close = core.Signal(QtWidgets.QWidget, str, QtGui.QIcon)
 
-    def __init__(self, name, widget):
+    def __init__(self, name: str, widget: QtWidgets.QWidget):
         super().__init__(None)
 
         self.set_id(name)
@@ -256,7 +318,7 @@ class DetachedTab(widgets.MainWindow):
     #  If the window is closed, emit the on_close and give the
     #  content widget back to the DetachableTabWidget
     def closeEvent(self, event):
-        self.on_close.emit(self.widget, self.id, self.windowIcon())
+        self.on_close.emit(self.widget, self.get_id(), self.windowIcon())
 
 
 if __name__ == "__main__":
@@ -267,4 +329,4 @@ if __name__ == "__main__":
     widget_2 = widgets.Widget()
     tab_widget.add_tab(widget_2, "Test 2")
     tab_widget.show()
-    app.exec_()
+    app.main_loop()
